package com.kaixed.kchat.data.repository

import com.kaixed.kchat.data.local.box.ObjectBox.getBoxStore
import com.kaixed.kchat.data.local.entity.Contact
import com.kaixed.kchat.data.local.entity.Contact_
import com.kaixed.kchat.data.model.friend.FriendRequestItem
import com.kaixed.kchat.data.model.search.SearchUser
import com.kaixed.kchat.network.ApiCall.apiCall
import com.kaixed.kchat.network.RetrofitClient
import com.kaixed.kchat.utils.ConstantsUtil.getUsername
import com.kaixed.kchat.utils.Pinyin4jUtil
import com.kaixed.kchat.utils.handle.ContactUtil
import io.objectbox.Box

class ContactRepository {

    private val contactApiService = RetrofitClient.contactApiService

    private val contactBox: Box<Contact> by lazy {
        getBoxStore().boxFor(Contact::class.java)
    }

    // 获取联系人请求列表
    suspend fun getContactRequestList(username: String): Result<List<FriendRequestItem>?> {
        return apiCall(
            apiCall = { contactApiService.getContactRequestList(username) },
            errorMessage = "获取好友申请列表失败"
        )
    }

    // 接受联系人请求
    suspend fun acceptContactRequest(
        username: String,
        contactId: String,
        remark: String
    ): Result<Contact?> {
        return apiCall(
            apiCall = { contactApiService.acceptContactRequest(contactId, username, remark) },
            errorMessage = "添加好友失败"
        ).onSuccess {
            it?.let {
                ContactUtil.handleContact(it)
            }
        }
    }

    // 删除联系人
    suspend fun deleteContact(
        username: String,
        contactId: String,
    ): Result<String?> {
        return apiCall(
            apiCall = { contactApiService.deleteContact(username, contactId) },
            errorMessage = "删除好友失败"
        ).onSuccess {
            val con =
                contactBox.query(Contact_.username.equal(contactId)).build().findFirst()
            contactBox.remove(con!!)
        }
    }

    // 添加联系人
    suspend fun addContact(contactId: String, message: String): Result<String?> {
        return apiCall(
            apiCall = { contactApiService.addContact(contactId, getUsername(), message) },
            errorMessage = "添加联系人失败"
        )
    }

    // 搜索联系人
    suspend fun searchContact(username: String): Result<SearchUser?> {
        return apiCall(
            apiCall = { contactApiService.searchContact(username) },
            errorMessage = "搜索用户失败"
        )
    }

    // 获取联系人列表
    suspend fun getContactList(username: String): Result<List<Contact>?> {
//        return safeApiCall(
//            apiCall = { contactApiService.getContactList(username) },
//            errorMessage = "获取好友列表失败"
//        ).onSuccess {
//
//        }
        return try {
            val response = contactApiService.getContactList(username)
            if (response.isSuccess()) {
                val searchUsers = response.getResponseData()
                searchUsers?.let {

                    val uiFriendList = searchUsers.map { networkItem ->
                        mapToFriendItem(networkItem) // 数据转换
                    }.sortedWith(::compareContacts)

                    // 设置是否显示分组头
                    uiFriendList.forEachIndexed { index, item ->
                        item.showHeader =
                            (index == 0 || item.quanpin?.get(index) != uiFriendList[index - 1].quanpin?.get(
                                index - 1
                            ))
                    }

                    Result.success(uiFriendList)
                } ?: Result.failure(Exception("当前没有好友"))
            } else {
                Result.failure(Exception(response.getResponseMsg()))
            }
        } catch (e: Exception) {
            Result.failure(e)
        }
    }

    suspend fun setRemark(userId: String, contactId: String, remark: String): Result<String?> {
        return apiCall(
            apiCall = { contactApiService.setRemark(userId, contactId, remark) },
            errorMessage = "设置备注失败"
        ).onSuccess {
            val con = contactBox.query(Contact_.username.equal(contactId)).build().findFirst()
            if (con != null) {
                con.remark = remark
                con.remarkquanpin = Pinyin4jUtil.toPinyin(remark)
                contactBox.put(con)
            }
        }
    }

    private fun mapToFriendItem(response: Contact): Contact {
        val pinyin = Pinyin4jUtil.toPinyin(response.nickname)
        response.remark?.let {
            response.remarkquanpin = Pinyin4jUtil.toPinyin(it)
        }
        response.quanpin = pinyin
        return response
    }

    private fun compareContacts(c1: Contact, c2: Contact): Int {
        val str1 = c1.remarkquanpin ?: c1.quanpin ?: c1.nickname
        val str2 = c2.remarkquanpin ?: c2.quanpin ?: c2.nickname

        val type1 = when {
            str1.matches(Regex("[a-zA-Z]+")) -> 1
            str1.matches(Regex("[0-9]+")) -> 2
            else -> 3
        }

        val type2 = when {
            str2.matches(Regex("[a-zA-Z]+")) -> 1
            str2.matches(Regex("[0-9]+")) -> 2
            else -> 3
        }

        return if (type1 != type2) {
            type1 - type2
        } else {
            str1.compareTo(str2)
        }
    }
}
