package com.kaixed.kchat.ui.activity

import android.Manifest
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Bundle
import android.provider.MediaStore
import android.widget.Toast
import androidx.activity.enableEdgeToEdge
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.bumptech.glide.Glide
import com.kaixed.kchat.data.local.box.ObjectBox.getBoxStore
import com.kaixed.kchat.data.local.entity.UserInfo
import com.kaixed.kchat.data.local.entity.UserInfo_
import com.kaixed.kchat.databinding.ActivityProfileDetailBinding
import com.kaixed.kchat.ui.base.BaseActivity
import com.kaixed.kchat.ui.widget.LoadingDialogFragment
import com.kaixed.kchat.utils.Constants.AVATAR_URL
import com.kaixed.kchat.utils.Constants.MMKV_USER_SESSION
import com.kaixed.kchat.utils.ConstantsUtil.getUsername
import com.kaixed.kchat.viewmodel.UserViewModel
import com.tencent.mmkv.MMKV
import io.objectbox.Box
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.MultipartBody
import okhttp3.RequestBody.Companion.asRequestBody
import java.io.File


class ProfileDetailActivity : BaseActivity<ActivityProfileDetailBinding>() {

    private val userInfoBox: Box<UserInfo> by lazy { getBoxStore().boxFor(UserInfo::class.java) }

    private val userSessionMMKV by lazy { MMKV.mmkvWithID(MMKV_USER_SESSION) }

    private val userViewModel: UserViewModel by viewModels()

    companion object {
        private const val TAG = "ProfileDetailActivity"

        // 定义请求码常量
        private const val REQUEST_CODE_PERMISSION = 1001
    }

    private val username: String by lazy { getUsername() }

    // 使用 ActivityResultLauncher 来注册一个结果处理器
    private lateinit var getImageLauncher: ActivityResultLauncher<Intent>

    override fun inflateBinding(): ActivityProfileDetailBinding {
        return ActivityProfileDetailBinding.inflate(layoutInflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()

        setListener()
        updateContent(username)

        getImageLauncher =
            registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
                if (result.resultCode == RESULT_OK) {
                    val selectedImageUri: Uri? = result.data?.data
                    selectedImageUri?.let {
                        updateAvatar(it)
                    }
                } else {
                    Toast.makeText(this, "未选择图片", Toast.LENGTH_SHORT).show()
                }
            }
    }

    override fun initData() {

    }

    private fun updateAvatar(uri: Uri) {
        val filePath = getPathFromUri(uri)

        val loadingDialogFragment = LoadingDialogFragment.newInstance("正在上传")

        loadingDialogFragment.showLoading(supportFragmentManager)

        userViewModel.uploadAvatarResult.observe(this) { result ->
            result.onSuccess {
                Glide.with(this).downloadOnly().load(it).submit()
                userSessionMMKV.putString(AVATAR_URL, it)
                toast("上传成功")
                binding.ciAvatar.setItemIcon(uri)
            }

            result.onFailure {
                toast("上传失败")
            }
        }
        val file = File(filePath!!)
        userViewModel.uploadAvatar(username = getUsername(), file = prepareFilePart(file))
        loadingDialogFragment.dismissLoading()
    }

    private fun prepareFilePart(file: File): MultipartBody.Part {
        val requestFile = file.asRequestBody("application/octet-stream".toMediaTypeOrNull())
        return MultipartBody.Part.createFormData("file", file.name, requestFile)
    }


    private fun getPathFromUri(uri: Uri): String? {
        val projection = arrayOf(MediaStore.Images.Media.DATA)
        val cursor = contentResolver.query(uri, projection, null, null, null)
        cursor?.use {
            val columnIndex = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA)
            if (cursor.moveToFirst()) {
                return cursor.getString(columnIndex)
            }
        }
        return null
    }

    private fun openGallery() {
        val intent = Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI)
        getImageLauncher.launch(intent)
    }


    override fun onRequestPermissionsResult(
        requestCode: Int, permissions: Array<out String>, grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == REQUEST_CODE_PERMISSION) {
            if (checkPermission()) {
                openGallery()
            } else {
                ActivityCompat.requestPermissions(
                    this,
                    arrayOf(Manifest.permission.READ_EXTERNAL_STORAGE),
                    REQUEST_CODE_PERMISSION
                )
            }
        }
    }

    private fun checkPermission(): Boolean {
        return ContextCompat.checkSelfPermission(
            this,
            Manifest.permission.READ_EXTERNAL_STORAGE
        ) == PackageManager.PERMISSION_GRANTED
    }

    private fun setListener() {
        binding.ciAvatar.setOnClickListener {
            checkPermission()
            openGallery()
        }

        binding.ciNickname.setOnClickListener {
            startActivity(
                Intent(this, RenameActivity::class.java)
            )
        }
    }

    override fun onResume() {
        super.onResume()
        updateContent(username)
    }

    private fun updateContent(username: String) {
        val userInfo = userInfoBox.query(UserInfo_.username.equal(username)).build().findFirst()
        if (userInfo != null) {
            binding.ciNickname.setItemDesc(userInfo.nickname)
            binding.ciAvatar.setItemIcon(userInfo.avatarUrl)
        }
    }
}
