package com.kaixed.kchat.ui.fragment

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.kaixed.kchat.data.local.entity.Contact
import com.kaixed.kchat.databinding.FragmentContactBinding
import com.kaixed.kchat.ui.adapter.FriendListAdapter
import com.kaixed.kchat.ui.base.BaseFragment
import com.kaixed.kchat.utils.ConstantsUtil.getUsername
import com.kaixed.kchat.viewmodel.ContactViewModel


class ContactFragment : BaseFragment<FragmentContactBinding>() {

    private val contactViewModel: ContactViewModel by viewModels()

    private var loading = false

    private lateinit var context: Context

    private val friendAdapter: FriendListAdapter by lazy {
        FriendListAdapter(mutableListOf(), requireContext())
    }

    companion object {
        private const val TAG = "ContactFragment"
    }

    override fun inflateBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): FragmentContactBinding {
        return FragmentContactBinding.inflate(inflater, container, false)
    }

    private val mReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val update = intent.getBooleanExtra("update", false)  // 获取单一的 boolean 类型值
            if (update) {
                loadData()
            }
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        context = requireContext()

        loadData()

        loadFriendRequest()

        val filter = IntentFilter("com.kaixed.kchat.broadcast.UPDATE_CONTACT_LIST")
        LocalBroadcastManager.getInstance(requireActivity()).registerReceiver(mReceiver, filter)
    }

    override fun onResume() {
        super.onResume()
        loadFriendRequest()
    }

    override fun onStart() {
        super.onStart()
        loadFriendRequest()
    }

    override fun onDestroy() {
        super.onDestroy()
        LocalBroadcastManager.getInstance(requireActivity()).unregisterReceiver(mReceiver);
    }

    private fun loadFriendRequest() {
        contactViewModel.contactRequestListResult.observe(viewLifecycleOwner) { result ->
            result.onSuccess {
                val items = result.getOrNull()?.toMutableList() ?: emptyList()
                if (items.isNotEmpty()) {
                    friendAdapter.items[0].apply {
                        remark =
                            "${items[items.size - 1].nickname}(${items[items.size - 1].message})"
                        avatarUrl = items[items.size - 1].avatarUrl
                    }
                    friendAdapter.notifyItemChanged(0)
                }
            }
            result.onFailure { }
        }
        contactViewModel.getContactRequestList(getUsername())
    }



    private fun loadData() {
        loading = true
        binding.tvLoading.visibility = View.VISIBLE
        binding.recycleFriendList.visibility = View.INVISIBLE

        val contacts = contactViewModel.loadFriendListInDb()
        val allItems = mutableListOf<Contact>().apply {
            addAll(getDefaultItems())
            addAll(contacts)
        }
        friendAdapter.updateData(allItems)
        binding.tvLoading.visibility = View.GONE
        binding.recycleFriendList.visibility = View.VISIBLE
        loading = false
        setupRecyclerView()
    }

    private fun setupRecyclerView() {
        binding.recycleFriendList.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        binding.recycleFriendList.adapter = friendAdapter
    }

    private fun getDefaultItems(): List<Contact> {
        return listOf(
            Contact(username = "", nickname = "新的朋友"),
            Contact(username = "", nickname = "群聊"),
            Contact(username = "", nickname = "标签"),
            Contact(username = "", nickname = "公众号"),
            Contact(username = "", nickname = "服务号")
        )
    }
}
