package com.kaixed.kchat.ui.widget


import android.animation.AnimatorSet
import android.animation.ArgbEvaluator
import android.animation.ValueAnimator
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.util.AttributeSet
import android.util.DisplayMetrics
import android.view.View
import android.view.animation.AccelerateInterpolator
import androidx.annotation.Nullable
import com.kaixed.kchat.R

class SwitchButton @JvmOverloads constructor(
    context: Context,
    @Nullable attrs: AttributeSet? = null,
    defStyleAttr: Int = 0,
    defStyleRes: Int = 0
) : View(context, attrs, defStyleAttr, defStyleRes) {

    companion object {
        /**
         * 背景动画的延时与持续时间
         */
        private const val BG_ANIMATOR_START_DELAY = 0
        private const val BG_ANIMATOR_DURATION = 200

        /**
         * dp 转 px 的工具方法
         */
        fun dip2px(context: Context, dipValue: Float): Int {
            val scale = context.resources.displayMetrics.density
            return (dipValue * scale + 0.5f).toInt()
        }

        fun getDisplayMetrics(context: Context): DisplayMetrics {
            return context.resources.displayMetrics
        }
    }

    // 属性变量
    private var mCheckedBg: Int
    private var mUncheckedBg: Int
    private var mCurrentBoundBg: Int
    private var mSmallCircleMargin: Int
    private val mBgPaint = Paint(Paint.ANTI_ALIAS_FLAG).apply { style = Paint.Style.FILL }
    private val mSmallCirclePaint = Paint(Paint.ANTI_ALIAS_FLAG).apply {
        style = Paint.Style.FILL
        color = Color.WHITE
    }
    private var mDefaultWidth: Int
    private var mDefaultHeight: Int
    private var mSmallCircleRadius = 0
    private var mWidth = 0
    private var mHeight = 0
    private var mBoundRadius = 0f
    private var mBoundRect: RectF? = null
    private var mSmallCircleCenterX = 0
    private var mSmallCircleCenterY = 0
    private var isChecked = false
    private var mCheckedChangeListener: OnCheckedChangeListener? = null
    private var mSmallCircleStartX = 0
    private var mSmallCircleEndX = 0
    private val mArgbEvaluator = ArgbEvaluator()
    private var mOffAnimatorSet: AnimatorSet? = null
    private var mOpenAnimatorSet: AnimatorSet? = null

    init {
        val typedArray = context.obtainStyledAttributes(
            attrs,
            R.styleable.SwitchButton,
            defStyleAttr,
            defStyleRes
        )
        mCheckedBg = typedArray.getColor(R.styleable.SwitchButton_sb_checked_bg, Color.BLUE)
        mUncheckedBg = typedArray.getColor(R.styleable.SwitchButton_sb_unchecked_bg, Color.GRAY)
        isChecked = typedArray.getBoolean(R.styleable.SwitchButton_sb_checked, false)
        mSmallCircleMargin = typedArray.getDimension(
            R.styleable.SwitchButton_sb_circle_bg_margin,
            dip2px(context, 3f).toFloat()
        ).toInt()
        typedArray.recycle()

        mCurrentBoundBg = if (isChecked) mCheckedBg else mUncheckedBg
        mDefaultWidth = dip2px(context, 50f)
        mDefaultHeight = dip2px(context, 30f)
        init()
    }

    private fun init() {
        setOnClickListener {
            if (isChecked) {
                if (mOffAnimatorSet?.isRunning == true) return@setOnClickListener
                off()
            } else {
                if (mOpenAnimatorSet?.isRunning == true) return@setOnClickListener
                open()
            }
            isChecked = !isChecked
            mCheckedChangeListener?.onCheckedChanged(this, isChecked)
        }
    }

    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)
        mWidth = w
        mHeight = h
        mSmallCircleRadius = (mHeight - mSmallCircleMargin * 2) / 2
        mBoundRadius = (mHeight / 2).toFloat()
        mSmallCircleCenterY = mHeight / 2
        mSmallCircleStartX = mWidth / 4
        mSmallCircleEndX = (mWidth / 4) * 3
        updateCheckStatus()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val heightMode = MeasureSpec.getMode(heightMeasureSpec)
        val heightSize = MeasureSpec.getSize(heightMeasureSpec)
        val widthMode = MeasureSpec.getMode(widthMeasureSpec)
        val widthSize = MeasureSpec.getSize(widthMeasureSpec)

        val resultWidth: Int
        val resultHeight: Int

        if (widthMode == MeasureSpec.EXACTLY || heightMode == MeasureSpec.EXACTLY) {
            resultWidth = widthSize
            resultHeight = heightSize
        } else {
            resultWidth = mDefaultWidth
            resultHeight = mDefaultHeight
        }
        setMeasuredDimension(resultWidth, resultHeight)
    }

    override fun onDraw(canvas: Canvas) {
        drawBg(canvas, mBgPaint)
        drawCircle(canvas, mSmallCirclePaint)
    }

    private fun drawBg(canvas: Canvas, paint: Paint) {
        mBgPaint.color = mCurrentBoundBg
        if (mBoundRect == null) {
            mBoundRect = RectF(0f, 0f, mWidth.toFloat(), mHeight.toFloat())
        }
        canvas.drawRoundRect(mBoundRect!!, mBoundRadius, mBoundRadius, paint)
    }

    private fun drawCircle(canvas: Canvas, paint: Paint) {
        canvas.drawCircle(
            mSmallCircleCenterX.toFloat(),
            mSmallCircleCenterY.toFloat(),
            mSmallCircleRadius.toFloat(),
            paint
        )
    }

    private fun open() {
        val startX = mSmallCircleStartX
        val endX = mSmallCircleEndX

        val openAnimator = ValueAnimator.ofInt(startX, endX).apply {
            duration = 300
            addUpdateListener { animation ->
                mSmallCircleCenterX = animation.animatedValue as Int
                invalidate()
            }
        }

        val bgAnimator = ValueAnimator.ofFloat(0f, 1f).apply {
            startDelay = BG_ANIMATOR_START_DELAY.toLong()
            duration = BG_ANIMATOR_DURATION.toLong()
            interpolator = AccelerateInterpolator()
            addUpdateListener { animation ->
                val fraction = animation.animatedValue as Float
                mCurrentBoundBg = mArgbEvaluator.evaluate(fraction, mUncheckedBg, mCheckedBg) as Int
                invalidate()
            }
        }

        mOpenAnimatorSet = AnimatorSet().apply {
            playTogether(openAnimator, bgAnimator)
            start()
        }
    }

    private fun off() {
        val startX = mSmallCircleEndX
        val endX = mSmallCircleStartX

        val offAnimator = ValueAnimator.ofInt(startX, endX).apply {
            duration = 200
            addUpdateListener { animation ->
                mSmallCircleCenterX = animation.animatedValue as Int
                invalidate()
            }
        }

        val bgAnimator = ValueAnimator.ofFloat(0f, 1f).apply {
            startDelay = BG_ANIMATOR_START_DELAY.toLong()
            duration = BG_ANIMATOR_DURATION.toLong()
            interpolator = AccelerateInterpolator()
            addUpdateListener { animation ->
                val fraction = animation.animatedValue as Float
                mCurrentBoundBg = mArgbEvaluator.evaluate(fraction, mCheckedBg, mUncheckedBg) as Int
                invalidate()
            }
        }

        mOffAnimatorSet = AnimatorSet().apply {
            playTogether(offAnimator, bgAnimator)
            start()
        }
    }

    fun setChecked(checked: Boolean) {
        isChecked = checked
        updateCheckStatus()
        postInvalidate()
    }

    private fun updateCheckStatus() {
        if (isChecked) {
            mSmallCircleCenterX = mSmallCircleEndX
            mCurrentBoundBg = mCheckedBg
        } else {
            mSmallCircleCenterX = mSmallCircleStartX
            mCurrentBoundBg = mUncheckedBg
        }
    }

    fun isChecked(): Boolean = isChecked

    fun setOnCheckedChangeListener(listener: OnCheckedChangeListener) {
        mCheckedChangeListener = listener
    }

    interface OnCheckedChangeListener {
        fun onCheckedChanged(button: SwitchButton, isChecked: Boolean)
    }
}
